<?php

namespace App\Http\Controllers;

use App\Models\Comment;
use App\Models\Like;
use App\Models\Post;
use App\Models\Topic;
use App\Models\User;
use App\Notifications\CustomNotification;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Storage;
use Inertia\Inertia;
use Parsedown;

class PostController extends Controller
{
    public function index()
    {
        return Inertia::render('Admin/Posts')
            ->title('Posts');
    }

    public function create()
    {
        return Inertia::render('Admin/SavePost', ['topics' => Topic::all()])
            ->title('Create Post');
    }

    public function update(Request $request)
    {
        $post = Post::where('id', $request->id)->with('topic')->first();

        if (!$post) {
            abort(404);
        }

        return Inertia::render('Admin/SavePost', [
            'post' => $post,
            'topics' => Topic::all(),
        ])
            ->title('Update Post ' . $post->title);
    }

    public static function save(Request $request)
    {

        $request->validate([
            'title' => 'required|max:255',
            'slug' => 'required|unique:posts,slug,' . $request->post_id,
            'content' => 'required',
            'topic' => 'nullable|exists:topics,id',
            'visibility' => 'nullable|in:public,private',
            'cover_image' => 'nullable|image',
        ]);

        $post = new Post();
        $author_id = $request->user()->id;

        if ($request->post_id) {
            $post = Post::where('id', $request->post_id)->where('author_id', $author_id)->first();
            if (!$post) {
                abort(404);
            }
            if ($request->cover_image) {
                Storage::disk('public')->delete('cover/' . $post->cover_image);
            }
        }
        $post->title = $request->title;
        $post->slug = $request->slug;
        $post->content = $request->content;
        $post->topic_id = $request->topic;
        $post->author_id = $author_id;
        $post->visibility = $request->visibility;

        if ($request->cover_image) {
            $post->cover_image = $post->slug . time() . "." . $request->cover_image->extension();
        }
        if ($post?->save()) {
            if ($request->cover_image) {
                $request->cover_image->storeAs(
                    'cover', $post->cover_image, 'public'
                );
            }
            return back()->with('message', 'Saved');
        }

    }

    public function deleteCoverImage(Request $request)
    {
        $post = Post::find($request->post_id);
        $cover_image = $post->cover_image;
        $post->cover_image = null;
        if ($post->save()) {
            Storage::disk('public')->delete('cover/' . $cover_image);
        }
    }

    public static function delete(Request $request)
    {

        $author_id = $request->user()->id;

        $post = Post::where('id', $request->post_id)->when($request->user()->role != 1, function ($q) use ($author_id) {
            $q->where('author_id', $author_id);
        })->first();

        if ($post?->delete()) {
            return back()->with('message', 'Deleted');
        }
    }

    public static function get(Request $request)
    {
        $data = Post::search($request->key);

        $data = $data->query(function ($query) use ($request) {
            $query->when($request->forusers, function ($q) {
                $q->where('visibility', 'public');
            });

            if ($request->topic) {
                $query->whereHas('topic', function ($q) use ($request) {
                    $q->where('name', $request->topic);
                });
            }
            $query->with(['topic', 'likes', 'comments']);
        })->paginate(1);

        return $data;
    }

    public function single(Request $request)
    {

        $post = Post::where('slug', $request->slug)->where('visibility', 'public')->with(['topic', 'likes'])->first();
        if (!$post) {
            abort(404);
        }

        $relatedByTopic = Post::where('topic_id', $post->topic_id)
            ->where('id', '!=', $post->id)
            ->where('visibility', 'public')
            ->limit(5);

        $relatedByTime = Post::where('id', '!=', $post->id)
            ->where('visibility', 'public')
            ->orderByRaw('ABS(TIMESTAMPDIFF(SECOND, created_at, ?))', [$post->created_at])
            ->limit(5);

        $relatedPosts = $relatedByTopic->union($relatedByTime)->get();

        $Parsedown = new Parsedown();
        $content = $Parsedown->text($post->content);
        $comments = Comment::where('post_id', $post->id)->with(['replies.user', 'user'])->whereNull('parent_id')->get();
        $commentsCount = Comment::where('post_id', $post->id)->count();

        return Inertia::render('Post', compact('post', 'comments', 'commentsCount', 'relatedPosts'))
            ->title($post->title)
            ->description(substr(strip_tags($content), 0, 160) . ' ...');

    }

    public function like(Request $request)
    {
        $user = Auth::user();
        $post_id = $request->post_id;
        $like = Like::where('user_id', $user->id)->where('post_id', $post_id)->first();
        if (!$like) {
            $like = new Like();
            $like->user_id = $user->id;
            $like->post_id = $post_id;
            $like->save();
        } else {
            $like->delete();
        }

    }

    public function comment(Request $request)
    {
        $user = Auth::user();
        $Parsedown = new Parsedown();
        $post_id = $request->post_id;
        $post = Post::find($post_id);
        $comment_id = $request->comment_id;

        if ($comment_id) {
            $comment = Comment::where('id', $comment_id)->where('user_id', $user->id)->first();
            $comment->edited = true;
        } else {
            $comment = new comment();
            $comment->user_id = $user->id;
            $comment->post_id = $post_id;
            $comment->parent_id = $request->parent_id;
        }

        $comment->text = $request->text;
        if ($comment->save()) {
            if ($comment->parent_id) {
                $parentComment = Comment::find($comment->parent_id);
                $toUser = User::find($parentComment->user_id);
                $title = $user->name . ' replied to your comment';
            } else {
                $toUser = User::where('role', '1')->first();
                $title = $user->name . ' commented on your post:' . $post->title;
            }
            $body = substr(strip_tags($Parsedown->text($comment->text)), 0, 100);
            $action = '/posts/' . $post->slug . '/#' . $comment->id;
            $toUser->notify(new CustomNotification($title, $body, $action, $image = null));
        }

    }

    public function deleteComment(Request $request)
    {
        $user = Auth::user();
        if ($user->role = 1) {
            $comment = Comment::find($request->comment_id);
        } else {
            $comment = Comment::where('id', $request->comment_id)->where('user_id', $user->id)->first();
        }
        if ($comment) {
            $comment->delete();
        }
    }

    public function getComments(Request $request)
    {
        $post_id = $request->post_id;

        return Comment::where('post_id', $post_id)->with(['replies.user', 'user'])->whereNull('parent_id')->paginate(10);
    }

}
