<?php

use App\Http\Controllers\MediaController;
use App\Http\Controllers\NotificationController;
use App\Http\Controllers\PostController;
use App\Http\Controllers\ProfileController;
use App\Http\Controllers\ProjectController;
use App\Http\Controllers\TopicController;
use App\Http\Controllers\UserController;
use App\Http\Middleware\AdminMiddleware;
use App\Models\Post;
use App\Models\Topic;
use App\Models\User;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Route;
use Inertia\Inertia;

/*
|--------------------------------------------------------------------------
| Web Routes
|--------------------------------------------------------------------------
|
| Here is where you can register web routes for your application. These
| routes are loaded by the RouteServiceProvider within a group which
| contains the "web" middleware group. Now create something great!
|
 */

function getLogoUrl()
{
    $path = '/storage/logo/';
    $directory = public_path($path);

    // Get the list of files in the directory
    $files = scandir($directory);

    // Remove "." and ".." entries
    $files = array_diff($files, array('.', '..'));
    return $path . array_values($files)[0];
}

Route::get(getLogoUrl(), function () {

    return getLogoUrl();

})->name('logoUrl');

Route::get('/', function (Request $request) {
    $initialposts = PostController::get($request);
    $params = $request;
    $topics = Topic::all();
    return Inertia::render('Posts', compact('initialposts', 'params', 'topics'))->title('All posts')->description('CodeGleam: Your go-to hub for insightful articles on web development. Dive into our comprehensive resources covering the latest trends, tutorials, and tips, curated to enhance your web development skills. Stay updated with expert insights and stay ahead in the ever-evolving world of web development.');
})->name('home');

Route::post('/posts', function (Request $request) {
    $data = PostController::get($request);
    return response()->json($data);
})->name('posts');

Route::get('/about', function (Request $request) {
    return Inertia::render('About')
        ->title('About')
        ->description("Discover the story behind CodeGleam.in – delve into our journey, mission, and the team behind our passion for empowering the web development community. Learn about Joydeep Bhowmik, the driving force steering our commitment to sharing valuable insights and expertise in the world of web development.");
})->name('about');

Route::get('/privacy-policy', function (Request $request) {
    return Inertia::render('PrivacyPolicy')
        ->title('Privacy Policy')
        ->description("Discover how CodeGleam.in prioritizes your privacy and handles your information. Our comprehensive privacy policy outlines how we collect, use, and protect your data, ensuring transparency and security in our commitment to safeguarding your online experience.");
})->name('pp');

Route::post('/search', [PostController::class, 'get'])->name('search');

Route::get('/posts/{slug}/{hash?}', [PostController::class, 'single'])->name('post');

Route::prefix('admin')
    ->middleware(['auth', 'verified', AdminMiddleware::class])
    ->group(function () {

        Route::get('/dashboard', fn() => Inertia::render('Dashboard', [
            'users' => User::count(),
            'posts' => Post::count(),
            'topics' => Topic::count(),
        ])->title('Dashboard'))
            ->name('admin.dashboard');

        Route::prefix('posts')->group(function () {

            Route::get('/', [PostController::class, 'index'])
                ->name('admin.posts');

            Route::get('/create', [PostController::class, 'create'])
                ->name('admin.posts.create');

            Route::get('/edit/{id}', [PostController::class, 'update'])
                ->name('admin.posts.update');

            Route::post('/save', [PostController::class, 'save'])
                ->name('admin.posts.save');

            Route::post('/delete', [PostController::class, 'delete'])
                ->name('admin.posts.delete');
        });

        Route::prefix('topics')->group(function () {

            Route::get('/', [TopicController::class, 'index'])
                ->name('admin.topics');

            Route::get('/create', [TopicController::class, 'create'])
                ->name('admin.topics.create');

            Route::get('/edit/{id}', [TopicController::class, 'update'])
                ->name('admin.topics.update');

            Route::post('/save', [TopicController::class, 'save'])
                ->name('admin.topics.save');

            Route::post('/delete', [TopicController::class, 'delete'])
                ->name('admin.topics.delete');
        });

        Route::prefix('projects')->group(function () {

            Route::get('/', [ProjectController::class, 'index'])
                ->name('admin.projects');

            Route::get('/create', [ProjectController::class, 'create'])
                ->name('admin.projects.create');

            Route::get('/edit/{id}', [ProjectController::class, 'update'])
                ->name('admin.projects.update');

            Route::post('/save', [ProjectController::class, 'save'])
                ->name('admin.projects.save');

            Route::post('/delete', [ProjectController::class, 'delete'])
                ->name('admin.projects.delete');
        });

        Route::get('/users', [UserController::class, 'index'])
            ->name('admin.users');

        Route::get('/send-notification',
            [NotificationController::class, 'create'])
            ->name('notifications.send');

        Route::get('/uploads',
            [MediaController::class, 'index'])
            ->name('uploads');

        Route::post('/upload-file',
            [MediaController::class, 'upload'])
            ->name('upload-file');

        Route::post('/delete-file',
            [MediaController::class, 'delete'])
            ->name('delete-file');

        Route::post('/delete-cover-image', [PostController::class, 'deleteCoverImage'])->name('deleteCoverImage');
    });

Route::middleware('auth')->group(function () {

    Route::get('/profile', [ProfileController::class, 'edit'])->name('profile.edit');

    Route::patch('/profile', [ProfileController::class, 'update'])->name('profile.update');

    Route::delete('/profile', [ProfileController::class, 'destroy'])->name('profile.destroy');

    Route::post('/like', [PostController::class, 'like'])->name('like');

    Route::post('/comment', [PostController::class, 'comment'])->name('comment');

    Route::post('/delete-comment', [PostController::class, 'comment'])->name('comment');

    Route::get('/notifications', [NotificationController::class, 'index'])->name('notifications');

});

Route::post('/comments', [PostController::class, 'deleteComment'])->name('deleteComment');

Route::get('/test', function (Request $request) {
    dd($request->getBaseUrl());
});

require __DIR__ . '/auth.php';
